/************************** Random.CPP ******************** AgF 2001-10-18 *
*  Random Number generator 'Mersenne Twister'
*
*
*
*  This random number generator is described in the article by
*
*  M. Matsumoto & T. Nishimura, in:
*
*  ACM Transactions on Modeling and Computer Simulation,
*
*  vol. 8, no. 1, 1998, pp. 3-30.
*
*
*
*  Experts consider this an excellent random number generator.
*
*
*
*  2002 A. Fog. GNU General Public License www.gnu.org/copyleft/gpl.html
*
****************************************************************************
*/


#include <math.h>

#include "mach.h"
#include "sierra.h"

#define N 351
#define M 175
#define R 19
#define TEMU 11
#define TEMS 7
#define TEMT 15
#define TEML 17

  // long constants cannot be defined as enum in 16-bit compilers:
#define MATRIX_A 0xE4BD75F5
#define TEMB     0x655E5280
#define TEMC     0xFFD58000



static unsigned long mt[N];                 // state vector
static int mti=N;


extern void set_seed (int seed) {
  unsigned long s = (unsigned long)seed;
  for (mti=0; mti<N; mti++) {
    s = s * 29943829 - 1;
    mt[mti] = s;}
  return;}

extern int genrandom () {
  // generate 32 random bits
  unsigned long y;

  if (mti >= N) {
    // generate N words at one time
    const unsigned long LOWER_MASK = (1u << R) - 1; // lower R bits
    const unsigned long UPPER_MASK = -1 << R;       // upper 32-R bits
    int kk, km;
    for (kk=0, km=M; kk < N-1; kk++) {
      y = (mt[kk] & UPPER_MASK) | (mt[kk+1] & LOWER_MASK);
      mt[kk] = mt[km] ^ (y >> 1) ^ (-(signed long)(y & 1) & MATRIX_A);
      if (++km >= N) km = 0;}

    y = (mt[N-1] & UPPER_MASK) | (mt[0] & LOWER_MASK);
    mt[N-1] = mt[M-1] ^ (y >> 1) ^ (-(signed long)(y & 1) & MATRIX_A);
    mti = 0;}

  y = mt[mti++];

  // Tempering (May be omitted):
  y ^=  y >> TEMU;
  y ^= (y << TEMS) & TEMB;
  y ^= (y << TEMT) & TEMC;
  y ^=  y >> TEML;

  return RAND_MASK&y;}

