(format t "~%~%This is the Sierra 2 system -- loaded sierra.lisp")

(defun help ()
  (format t "~%~%(setseed <n>)                change random seed")
  (format t "~%(setparams <copy> <cosmic>)  set error parameters")
  (format t "~%(status)                     current status of the run")
  (format t "~%(run)                        100,000,000 more instructions")
  (format t "~%(run <n>)                    <n> more instructions")
  (format t "~%(stop)                       stop running immediately")
  (format t "~%(help)                       this message")
  (format t "~%~%(rep)                        current status of population")
  (format t "~%(best)                       print ids of best genomes")
  (format t "~%(genomes <size>)             good genomes of <size>")
  (format t "~%(print-genome <id|gen>)      tells about genome's success")
  (format t "~%(print-code <id|gen>)        lists its code")
  (format t "~%~%(errors <bool>)              suppresses errors when <bool>==0")
  (format t "~%(dealloc <bool>)             clears memory when creature dies <bool>==0")
  (format t "~%~%(morehelp)                   other helpful procedures")
  (format t "~%"))

(help)

(defparameter *ancestor* (get-ancestor))

(defun print-genome (thing)
  (if (fixnump thing)
      (print-gen (get-genome thing))
    (print-gen thing)))

(defun print-code (thing)
  (if (fixnump thing)
      (print-cod (get-genome thing))
    (print-cod thing)))

(defun setparams (copy cosmic)
  (setparamsint copy cosmic)
  (format t "~%Copy errors 1 every ~d, cosmic rays 1 every ~d instructions."
	  copy
	  cosmic)
  't)

(defun setseed (n)
  (setseedint n)
  (format t "~%Random seed is ~d." n)
  't)

(format t "~%Current settings:")

(setparams 2000 500)

(setseed 255)

(defun run (&optional (more 100000000)) (runsierra more))

(defun status ()
  (let ((result (repstatus)))
    (if (= result 0)
	(format t "~&Not started")
      (if (> result 0)
	  (format t "~&Running at ~S" result)
	(format t "~&Idle at ~S" (- result)))))
  '())

(defun check-genome (g)
  (unless (check-ltype g "genome")
    (error "Not a valid genome: ~S" g)))

(defun genome-id (g)
  (check-genome g)
  (g-id g))

(defun genome-instances (g)
  (check-genome g)
  (g-instances g))

(defun genome-spawned (g)
  (check-genome g)
  (g-spawned g))

(defun genome-selfrep (g)
  (check-genome g)
  (g-selfrep g))

(defun genome-generations (g)
  (check-genome g)
  (g-generations g))

(defun genome-length (g)
  (check-genome g)
  (g-length g))

(defun genome-parent (g)
  (check-genome g)
  (g-parent g))

(defun morehelp ()
  (format t "~%~%(number-genomes)               total number of genomes")
  (format t "~%(get-genome <id>)              returns genome with <id>")
  (format t "~%(genome-id <genome>)           id of <genome>")
  (format t "~%(genome-instances <genome>)    how many of these existed")
  (format t "~%(genome-spawned <genome>)      how many of these reproduced")
  (format t "~%(genome-selfrep <genome>)      how many exact reproductions")
  (format t "~%(genome-generations <genome>)  how many generations to produce")
  (format t "~%(genome-length <genome>)       length of <genome>")
  (format t "~%(genome-parent <genome>)       parent genome of <genome>")
  (format t "~%"))

(format t "~%~%")
